#if !defined(_NetService_h__)
#define _NetService_h__

//////////////////////////////////////////////////////////////////////////////////

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//////////////////////////////////////////////////////////////////////////////////
//
//		CNetService Class
//
//////////////////////////////////////////////////////////////////////////////////
//
//		Written by  Jay Wheeler
//					EarthWalk Software
//
//		Version 1 - October, 2000.
//
//////////////////////////////////////////////////////////////////////////////////
//
//		CNetService is copyright  2000. EarthWalk Software.
//
//		This library is free software; you can redistribute it and/or
//		modify it under the terms of the GNU Lesser General Public
//		License as published by the Free Software Foundation; either
//		version 2.1 of the License, or (at your option) any later version.
//
//		This library is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//		Lesser General Public License for more details.
//
//		You should have received a copy of the GNU Lesser General Public
//		License along with this library; if not, write to the Free Software
//		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//////////////////////////////////////////////////////////////////////////////////

#include <winnetwk.h>

//////////////////////////////////////////////////////////////////////////////////
//
//		NOTE:  Project must also include the following libraries:
//					mpr.lib
//
//////////////////////////////////////////////////////////////////////////////////

typedef struct _OSVERSTRUCT
{
	union
	{
		OSVERSIONINFOEX		OSVerX;
		OSVERSIONINFO		OSVer;
	};
} OSVERSTRUCT;


//////////////////////////////////////////////////////////////////////////////////

#define MAX_ENUM_BUFFER			16000

//////////////////////////////////////////////////////////////////////////////////
//
//			NetService Error Codes
//
//////////////////////////////////////////////////////////////////////////////////

#define	NETSERVICE_NOERROR					0
#define NETSERVICE_OPEN_ENUM				NETSERVICE_NOERROR + 1
#define NETSERVICE_ENUM_RESOURCE			NETSERVICE_OPEN_ENUM + 1
#define NETSERVICE_ADDCONN_ERROR			NETSERVICE_ENUM_RESOURCE + 1
#define NETSERVICE_GET_OSVER				NETSERVICE_ADDCONN_ERROR + 1
#define NETSERVICE_GETUSER					NETSERVICE_GET_OSVER + 1
#define NETSERVICE_CANCELCONNECT			NETSERVICE_GETUSER + 1
#define	NETSERVICE_GETUNC_INFO				NETSERVICE_CANCELCONNECT + 1
#define	NETSERVICE_GETREMOTE_INFO			NETSERVICE_GETUNC_INFO + 1
#define NETSERVICE_GETPROVIDER				NETSERVICE_GETREMOTE_INFO + 1
#define NETSERVICE_CONN_DLG					NETSERVICE_GETPROVIDER + 1
#define	NETSERVICE_DISCONN_DLG				NETSERVICE_CONN_DLG + 1
#define NETSERVICE_MEM_ALLOC				NETSERVICE_DISCONN_DLG + 1
#define NETSERVICE_HEAP_ALLOC				NETSERVICE_MEM_ALLOC + 1
#define NETSERVICE_NETWORKINFO				NETSERVICE_HEAP_ALLOC + 1

//////////////////////////////////////////////////////////////////////////////////
//
//			Windows OS Types
//
//////////////////////////////////////////////////////////////////////////////////

#define WIN_UNKNOWN							0
#define WIN32_S								WIN_UNKNOWN + 1
#define WIN32_NT_PRO						WIN32_S + 1
#define WIN32_NT_SERVER						WIN32_NT_PRO + 1
#define WIN32_2000_PRO						WIN32_NT_SERVER + 1
#define WIN32_2000_SERVER					WIN32_2000_PRO + 1
#define WIN32_WINDOWS_95					WIN32_2000_SERVER + 1
#define WIN32_WINDOWS_98					WIN32_WINDOWS_95 + 1

//////////////////////////////////////////////////////////////////////////////////
//
//
//
//////////////////////////////////////////////////////////////////////////////////
typedef struct _UNCInfo
{
	union
	{
		UNIVERSAL_NAME_INFO		UNC_NameInfo;
		REMOTE_NAME_INFO		UNC_RemoteInfo;
	} Info;
} UNCInfo;

#define MaxUNCInfo			4000

//////////////////////////////////////////////////////////////////////////////////
//
//			CNetService Class
//
//////////////////////////////////////////////////////////////////////////////////
class CNetService
{

public:
							CNetService(DWORD p_MemSize=MAX_ENUM_BUFFER);
	virtual					~CNetService();

	//////////////////////////////////////////////////////////////////////////////

public:
	virtual bool			OpenEnum(DWORD p_Scope, 
									 DWORD p_Type, 
									 DWORD p_Usage, 
									 NETRESOURCE * p_pNetResource=NULL);
	virtual void			CloseEnum(void);

	virtual bool			EnumerateResource(DWORD p_Count);
	virtual NETRESOURCE *	EnumerateNext(void);

	virtual bool			GetUNCInfo(CString p_LocalPath);
	CString					UniversalName(void);

	CString					UNCRemoteServer(void);
	CString					UNCRemotePath(void);

	CString					GetProviderName(DWORD p_NetType);

	virtual bool			AddConnection(NETRESOURCE * p_pNetResource,
										  CString p_UserName,
										  CString p_Password,
										  bool p_Persistent=0);
	virtual bool			AddConnection(HWND p_hWnd,
								           NETRESOURCE * p_pNetResource,
										   CString p_UserName,
										   CString p_Password,
										   bool p_Persistent);

	virtual bool			NetConnect(DWORD   p_NetType,
									   CString p_Local,
									   CString p_Remote,
									   CString p_NetStack,
									   CString p_UserName,
									   CString p_Password,
									   bool    p_Persistent);
	virtual bool			NetConnect(HWND    p_hWnd,
										DWORD   p_NetType,
										CString p_Local,
										CString p_Remote,
										CString p_NetStack,
										CString p_UserName,
										CString p_Password,
										bool    p_Persistent);

	virtual bool			NetDisconnect(CString p_Local,
										  DWORD   p_DisconnectType,
										  bool	  p_ForceDisconnect);

	CString					NetUser(CString p_Remote);

	int						OSVersion(void);

	long					Result(void);
	bool					OpenState(void);

	DWORD					Error(void);
	CString					ErrorMessage(DWORD p_Error);

	DWORD					ServiceError(void);
	CString					ServiceErrorMessage(DWORD p_Error);

	DWORD					NetError(void);
	CString					NetErrorMessage(void);
	CString					NetErrorProvider(void);

	DWORD					ResourceDisplayType(void);

	bool					NetworkInformation(CString p_ProviderName);
	bool					NetworkInformation(CString p_ProviderName,
											   NETINFOSTRUCT * p_pNetInfoStruct);

	NETINFOSTRUCT			* GetNetInfo(void);
	DWORD					CNetService::GetNetStatus(void);	
	WORD					CNetService::GetNetType(void);
	DWORD					CNetService::GetNetInfoPrinters(void);
	DWORD					CNetService::GetNetInfoDrives(void);

protected:
	bool					GetNetError(int p_ServiceError);
private:
	LPVOID					GetMem(DWORD p_MemSize);
	void					FreeMem(LPVOID p_pMem);

	//////////////////////////////////////////////////////////////////////////////
	
public:

protected:
	long						g_Result;
	bool						g_Open;
	DWORD						g_Error;

	int							g_ServiceError;
	DWORD						g_Count;

	DWORD						g_NetError;
	CString						g_NetErrorMessage;
	CString						g_NetErrorProvider;

	CString						g_UserName;
	CString						g_ProviderName;

	HANDLE						g_hEnum;
	HANDLE						g_hMem;
	DWORD						g_MemSize;
	DWORD						g_EnumMemSize;

	NETRESOURCE					* g_pNetResource;
	NETINFOSTRUCT				g_NetInfoStruct;

	CString						g_UniversalName;
	CString						g_RemainingPath;
	CString						g_ConnectionName;

	int							g_WindowsType;

	DWORD						g_EnumIndex;
	DWORD						g_EnumBufferSize;
	char						* g_pEnumBuffer;

private:
};

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

#endif
